/*
    (C) 2004 Harry Sintonen
    $Id: setdate.c,v 1.1.1.1 2005/03/15 16:00:58 laire Exp $
 
    Desc: SetDate CLI command
    Lang: English
*/

/*
    NAME

        SetDate

    FORMAT

        SetDate (file | pattern) [(weekday)] [(date)] [(time)] [ALL]

    SYNOPSIS

        FILE/A,WEEKDAY,DATE,TIME,ALL/S

    LOCATION

        Workbench:c

    FUNCTION

        Changes the the date and time of the creation or last change of a
        file or directory. With option ALL, it changes the date and time of
        all files and directories (and files and subdirectories to those)
        matching the specified pattern.
        You may use the output from Date as input to SetDate.

    INPUTS

        FILE     --  File (or pattern) to change the date of.

        WEEKDAY  --  Specification of the day of the date. This is locale
                     sensitive, and you may use standard keywords as
                     'Tomorrow' and 'Yesterday' (in the language used, of
                     course).

        DATE     --  A date described according to the locale specification
                     of the currently used language.

        TIME     --  Time string in localized format.

        ALL      --  Recurse through subdirectories.

    RESULT

        Standard DOS return codes

    NOTES

    EXAMPLE

        SetDate #? ALL

        Sets the date for all files and directories in the current directory
        and its subdirectories to the current date.

    BUGS

    SEE ALSO

        Date

    INTERNALS

    HISTORY

        9.5.04  Piru    implemented. fixed ALL/S.
*/

#include <dos/datetime.h>
#include <dos/dosasl.h>
#include <exec/memory.h>

#include <proto/dos.h>
#include <proto/exec.h>

#include <sys/types.h>
void *memclr(void *,size_t);

#include "setdate_version.h"


#define USE_SOFTLINKFIX		1


static const char version[] = VERSTAG;

#define TEMPLATE "FILE/A,WEEKDAY,DATE,TIME,ALL/S"
enum { ARG_FILE = 0, ARG_WEEKDAY, ARG_DATE, ARG_TIME, ARG_ALL, NUMARGS };

int main(void)
{
	LONG               retval = RETURN_FAIL;
	struct Library    *SysBase = *((struct Library **) 4);
	struct DosLibrary *DOSBase;

	DOSBase = (struct DosLibrary *) OpenLibrary("dos.library", 37);
	if (DOSBase)
	{
		UBYTE stack_ap[sizeof(struct AnchorPath) + 3];
		struct AnchorPath *aPath;
		struct RDArgs  *rda;
		IPTR            args[NUMARGS];
		struct DateTime dt;
		LONG            error;
		UBYTE         **ptr;

		memclr(args, sizeof(args));

		rda = ReadArgs(TEMPLATE, args, NULL);
		if (!rda)
		{
			PrintFault(IoErr(), NULL);
			goto closedos;
		}

		/* Defauls to current date & time */
		DateStamp(&dt.dat_Stamp);
		dt.dat_Flags   = DTF_FUTURE;
		dt.dat_Format  = FORMAT_DOS;

		for (ptr = (UBYTE **) &args[ARG_WEEKDAY];
		     ptr <= (UBYTE **) &args[ARG_TIME];
		     ptr++)
		{
			UBYTE *str = *ptr;
			if (str)
			{
				dt.dat_StrDate = str;
				dt.dat_StrTime = NULL;
				if (!StrToDate(&dt))
				{
					dt.dat_StrDate = NULL;
					dt.dat_StrTime = str;
					if (!StrToDate(&dt))
					{
						PutStr("SetDate failed: Invalid WEEKDAY, DATE or TIME string!\n");
						goto freeargs;
					}
				}
			}
		}

		aPath = (struct AnchorPath *) (((ULONG) stack_ap + 3) & ~3);
		aPath->ap_Flags     = APF_DOWILD;
		aPath->ap_BreakBits = SIGBREAKF_CTRL_C;
		aPath->ap_Strlen    = 0;

		if (DOSBase->dl_lib.lib_Version > 50 ||
		    (DOSBase->dl_lib.lib_Version == 50 &&
		     DOSBase->dl_lib.lib_Revision >= 67))
		{
			aPath->ap_Strlen    |= APSF_EXTENDED;
			aPath->ap_Extended  = APEF_DontFollowSLinks;
		}

		for (error = MatchFirst((STRPTR) args[ARG_FILE], aPath);
		     !error;
		     error = MatchNext(aPath))
		{
			BPTR lock, olddirlock;
			LONG ret;

			if (aPath->ap_Info.fib_DirEntryType > 0 &&
#if USE_SOFTLINKFIX
			    aPath->ap_Info.fib_DirEntryType != ST_SOFTLINK &&
#endif
			    args[ARG_ALL])
			{
				if (!(aPath->ap_Flags & APF_DIDDIR))
				{
					aPath->ap_Flags |= APF_DODIR;

					/* This 'continue' makes MatchNext scan the entries in reverse
					 * order... Which is correct for SetFileDate, since setting
					 * entries in dir might set date of the parent directories.
					 * This way setting the dates should work properly in all
					 * cases.
					 */
					continue;
				}
				aPath->ap_Flags &= ~APF_DIDDIR;
			}

			if (error && error != ERROR_NO_MORE_ENTRIES)
			{
				break;
			}

			lock = DupLock(aPath->ap_Current->an_Lock);
			if (!lock && aPath->ap_Current->an_Lock)
			{
				error = IoErr();
				break;
			}

			olddirlock = CurrentDir(lock);

			#if 0
			Printf("SetFileDate: <%s>\n", (LONG) aPath->ap_Info.fib_FileName);
			Delay(1);
			#endif

			ret = SetFileDate(aPath->ap_Info.fib_FileName, &dt.dat_Stamp);
#if USE_SOFTLINKFIX
			if (!ret && aPath->ap_Info.fib_DirEntryType == ST_SOFTLINK)
			{
				ret = DOSTRUE;
			}
#endif
			if (!ret)
			{
				error = IoErr();
			}

			CurrentDir(olddirlock);
			UnLock(lock);

			if (!ret)
			{
				break;
			}
		}

		MatchEnd(aPath);

		if (error == ERROR_NO_MORE_ENTRIES)
		{
			retval = RETURN_OK;
		}
		else
		{
			PrintFault(error, "SetDate failed");

			retval = error == ERROR_BREAK ? RETURN_WARN : RETURN_FAIL;
		}

		freeargs:
		FreeArgs(rda);

		closedos:
		CloseLibrary((struct Library *) DOSBase);
	}

	return retval;
}
